/***********************************************************************/
/*                                                                     */
/*                               Pio (C)                               */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*               EPI Pomdapi, INRIA Paris - Rocquencourt               */
/*                                                                     */
/*  Copyright 2011-2014 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Bruno Verlyck <Bruno.Verlyck@inria.fr>                             */
/*                                                                     */
/***********************************************************************/

#include "common.h"
#include "debug_io.h"
#include <stdio.h>
#include <string.h>
#include "typed_value.h"
#include "communication.h"

service_name service_name_of_string (const char *name) {
  if (!strcmp (name, "Ok"))
    return COM_OK;
  else if (!strcmp (name, "Ko"))
    return COM_KO;
  else if (!strcmp (name, "Allo"))
    return COM_allo;
  else if (!strcmp (name, "Bye"))
    return COM_bye;
  else if (!strcmp (name, "Start"))
    return COM_start;
  else if (!strcmp (name, "Stop"))
    return COM_stop;
  else if (!strcmp (name, "WrongCommunication"))
    return COM_wrong_communication;
  else {
    static char format[] = "Unknown service kind %s";
    /* We'll truncate */
    char msg[sizeof (format) + 3 * sizeof ("WrongCommunication")];
    snprintf (msg, sizeof (msg), format, name);
    msg[sizeof (msg) - 1] = '\0';       /* does snprintf do this on overflow ? */
    raise_wrong_communication (msg);
  }
}

const char *string_of_service_name (service_name name) {
  switch (name) {
  case COM_OK:                  return "Ok";
  case COM_KO:                  return "Ko";
  case COM_allo:                return "Allo";
  case COM_bye:                 return "Bye";
  case COM_start:               return "Start";
  case COM_stop:                return "Stop";
  case COM_wrong_communication: return "WrongCommunication";
  default:
    assert (0);
    pio_fatal_error ("string_of_service_name: assertion failed.");
  }
}

COM_t *make_communication (com_tag tag) {
  COM_t *com = malloc (sizeof (COM_t));   /* XXX voir  allouer par paquets, mais free ? */
  if (com == NULL)
    failwith ("make_communication: out of memory");

  memset ((void *)com, 0, sizeof (*com)); /* realloc doesn't clear memory */
  com->com_tag = tag;
  return com;
}
