(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2005-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: communication.ml,v 1.7 2014/01/10 19:22:10 fclement Exp $ *)

type values = Typed_value.t array;;

type service_name =
   | Ok | Ko | Allo | Bye | Start | Stop | WrongCommunication
;;

type task_name = string;;

type t =
  | Phrase of values
  | Task of task_name * values
  | Result of values
  | Error of values
  | Service of service_name * values
;;
(** The general type for communications. *)

let service_name_of_string = function
  | "Ok" -> Ok
  | "Ko" -> Ko
  | "Allo" -> Allo
  | "Bye" -> Bye
  | "Start" -> Start
  | "Stop" -> Stop
  | "WrongCommunication" -> WrongCommunication
  | sname ->
      raise
        (Typed_value.Wrong_communication
           (Printf.sprintf "Unknown service kind %s" sname))
;;

let string_of_service_name = function
  | Ok -> "Ok"
  | Ko -> "Ko"
  | Allo -> "Allo"
  | Bye -> "Bye"
  | Start -> "Start"
  | Stop -> "Stop"
  | WrongCommunication -> "WrongCommunication"
;;
