(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: log.ml,v 1.6 2014/01/21 09:21:16 fclement Exp $ *)

type log_level =
  | Say
  | Log of int
  | Warn
  | Err
;;

let me = Filename.basename Sys.argv.(0);;

let verbosity =
  match
    try Some (Sys.getenv "SKLML_VERBOSITY") with _ -> None
  with
  | Some v ->
    let vi =
      try int_of_string v
      with Failure _ ->
        Printf.eprintf "%s: SKLML_VERBOSITY must be an integer.\n%!" me;
        0
    in vi
  | None -> 0
;;

let ksay_intern prefix suffix log_level fmt =
  let real_logger ch out =
    let ksuff s =
      Printf.fprintf ch "%s%s%!" s suffix; out
    in
    match prefix with
    | Some p ->
      Printf.ksprintf ksuff ("%s: " ^^ fmt) p
    | None ->
      Printf.ksprintf ksuff fmt
  and dummy_logger _ k =
    Printf.ksprintf (fun _ -> k) fmt
  in
  match log_level with
  | Say -> real_logger stdout
  | Err -> real_logger stderr
  | Warn ->
    if (-1) >= verbosity
    then dummy_logger stderr
    else real_logger stderr
  | Log i ->
    if i >= verbosity
    then dummy_logger stderr
    else real_logger stderr
;;

let say level fmt = ksay_intern (Some me) "\n" level fmt ();;

let progress level fmt =
  let final x = ksay_intern None "\n" level x () in
  ksay_intern (Some me) "" level fmt final
;;
