(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: skl_comm.ml,v 1.17 2014/01/21 09:21:16 fclement Exp $ *)

type atom_id = int;;

type color = int;;

type request =
   | Atom_req of Obj.t * atom_id * color
   | Loop_req of Iotree.ioatree * Iotree.ioatree
   | Pipe_req of Iotree.ioatree * Iotree.ioatree
;;

type port = int;;

type reply =
   | Success of Unix.sockaddr
   | Failure
;;

type iochans = IO.ic * IO.oc;;

let request_daemon (ic, oc) req =
  try
    IO.to_channel oc req true;
    let rep = IO.of_channel ic in
    Some rep
  with _ ->
    None
;;

let extract_request ic =
  IO.of_channel ic
;;

let send_reply oc rep =
  IO.to_channel oc rep true
;;

let get_tmp () =
  try Sys.getenv "TMP"
  with Not_found -> "/tmp"
;;

let dt = 0.01;;

let usleep dt =
  ignore (Unix.select [] [] [] dt)
;;

let open_skl_fifo b pid =
  let fname suff = Printf.sprintf "sklfifo%d_%s" pid suff in
  let path suff = Filename.concat (get_tmp ()) (fname suff) in
  let rec wait_for_file k path =
    if Sys.file_exists path
    then k path
    else begin usleep dt; wait_for_file k path end in
  if b then
    let ic = IO.openfifo_in (path "in") in
    let oc = wait_for_file IO.open_out (path "out") in
    (ic, oc)
  else
    let oc = wait_for_file IO.open_out (path "in") in
    let ic = IO.openfifo_in (path "out") in
    (ic, oc)
;;

let clean_skl_fifo pid =
  let fname suff = Printf.sprintf "sklfifo%d_%s" pid suff in
  let path suff = Filename.concat (get_tmp ()) (fname suff) in
  let remx s =
    let f = path s in
    if Sys.file_exists f then
    Sys.remove f
  in
  List.iter remx [ "in"; "out"; ]
;;
