CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright 2005-2014 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: fetch_memory_mapping.fpp,v 1.2 2014/01/10 19:22:10 fclement Exp $

C     
C     Routines to read the memory mapping.
C     
      
#include "pio_constants.h"
      
      integer function fetch_communication_type(sbr, io)
C     Fetch slot SLOT_COMMUNICATION_TYPE (Tc).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io
      
C     External function.
      integer rand_fetch_slot
      
      fetch_communication_type =
     +     rand_fetch_slot(sbr, io, SLOT_COMMUNICATION_TYPE)
      
      return
      end
      
      
      integer function fetch_communication_values_number(sbr, io)
C     Fetch slot SLOT_VALUES_NUMBER (Nv).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io
      
C     External function.
      integer rand_fetch_slot
      
      fetch_communication_values_number =
     +     rand_fetch_slot(sbr, io, SLOT_VALUES_NUMBER)
      
      return
      end
      
      
      integer function fetch_communication_name_begin(sbr, io)
C     Get the beginning in memory of the communication name.
C     Fetch slot SLOT_NAME_BEGIN (bn).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io
      
C     External function.
      integer rand_fetch_slot
      
      fetch_communication_name_begin =
     +     rand_fetch_slot(sbr, io, SLOT_NAME_BEGIN)
      
      return
      end
      
      
      integer function fetch_communication_name_end(sbr, io)
C     Get the end in memory of the communication name.
C     Fetch slot SLOT_NAME_END (en).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io
      
C     External function.
      integer rand_fetch_slot
      
      fetch_communication_name_end =
     +     rand_fetch_slot(sbr, io, SLOT_NAME_END)
      
      return
      end
      
      
      integer function fetch_communication_begin(sbr, io)
C     Get the beginning in memory of the communication values.
C     Fetch slot SLOT_COMMUNICATION_BEGIN (bc).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io
      
C     External function.
      integer rand_fetch_slot
      
      fetch_communication_begin =
     +     rand_fetch_slot(sbr, io, SLOT_COMMUNICATION_BEGIN)
      
      return
      end
      
      
      integer function fetch_communication_end(sbr, io)
C     Get the end in memory of the communication values.
C     Fetch slot SLOT_COMMUNICATION_END (ec).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io
      
C     External function.
      integer rand_fetch_slot
      
      fetch_communication_end =
     +     rand_fetch_slot(sbr, io, SLOT_COMMUNICATION_END)
      
      return
      end
      
      
      integer function fetch_communication_value_descriptor_address
     +     (sbr, io, i)
C     Get the value descriptor address of the ith value.
C     Fetch item i of slot vda.
C     Fetch slot (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, i

C     External function.
      integer rand_fetch_slot
      
      fetch_communication_value_descriptor_address =
     +     rand_fetch_slot(sbr, io, SLOT_VALUE_DESCRIPTOR_ADDRESS(i))
      
      return
      end
      
      
      integer function fetch_value_type(sbr, io, i)
C     Get the type of ith value.
C     Fetch slot SLOT_VALUE_TYPE of item i of slot vm.
C     Fetch slot (vda(i) + SLOT_VALUE_TYPE), where vda(i) is the value
C     in slot (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, i
      
C     Local variables.
      integer vdai, loc
      
C     External functions.
      integer fetch_communication_value_descriptor_address
      integer rand_fetch_slot
      
      vdai = fetch_communication_value_descriptor_address(sbr, io, i)
      loc = vdai + SLOT_VALUE_TYPE
      
      fetch_value_type = rand_fetch_slot(sbr, io, loc)
      
      return
      end
      
      
      integer function fetch_value_dimension(sbr, io, i)
C     Get the dimension of ith value.
C     Fetch slot SLOT_VALUE_DIMENSION of item i of slot vm.
C     Fetch slot (vda(i) + SLOT_VALUE_DIMENSION), where vda(i) is the
C     value in slot (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, i
      
C     Local variables.
      integer vdai, loc
      
C     External functions.
      integer fetch_communication_value_descriptor_address
      integer rand_fetch_slot
      
      vdai = fetch_communication_value_descriptor_address(sbr, io, i)
      loc = vdai + SLOT_VALUE_DIMENSION
      
      fetch_value_dimension = rand_fetch_slot(sbr, io, loc)
      
      return
      end
      
      
      integer function fetch_value_begin(sbr, io, i)
C     Get the beginning in memory of ith value.
C     Fetch slot SLOT_VALUE_BEGIN of item i of slot vm.
C     Fetch slot (vda(i) + SLOT_VALUE_BEGIN), where vda(i) is the value
C     in slot (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, i
      
C     Local variables.
      integer vdai, loc
      
C     External functions.
      integer fetch_communication_value_descriptor_address
      integer rand_fetch_slot
      
      vdai = fetch_communication_value_descriptor_address(sbr, io, i)
      loc = vdai + SLOT_VALUE_BEGIN
      
      fetch_value_begin = rand_fetch_slot(sbr, io, loc)
      
      return
      end
      
      
      integer function fetch_value_end(sbr, io, i)
C     Get the end in memory of ith value.
C     Fetch slot SLOT_VALUE_END of item i of slot vm.
C     Fetch slot (vda(i) + SLOT_VALUE_END), where vda(i) is the value in
C     slot (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, i
      
C     Local variables.
      integer vdai, loc
      
C     External functions.
      integer fetch_communication_value_descriptor_address
      integer rand_fetch_slot
      
      vdai = fetch_communication_value_descriptor_address(sbr, io, i)
      loc = vdai + SLOT_VALUE_END
      
      fetch_value_end = rand_fetch_slot(sbr, io, loc)
      
      return
      end
      
      
      subroutine fetch_value_sizes(sbr, io, i, sizev)
C     Get the sizes of ith value into sizev.
C     Fetch slots
C     SLOT_VALUE_SIZES_BEGIN..(SLOT_VALUE_SIZES_BEGIN + pv - 1)
C     of item i of slot vm into sizev, where pv is the dimension of the
C     value.
C     Fetch slots (vda(i) + SLOT_VALUE_SIZES_BEGIN)..
C     (vda(i) + SLOT_VALUE_SIZES_BEGIN + pv - 1) into sizev,
C     where vda(i) is the value in slot
C     (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1)
C     and pv the value in slot SLOT_VALUE_DIMENSION.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, i
      
C     Output argument.
      integer sizev(*)
      
C     Local variables.
      integer vdai, loc, pv, j
      
C     External functions.
      integer fetch_communication_value_descriptor_address
      integer fetch_value_dimension, rand_fetch_slot
      
      vdai = fetch_communication_value_descriptor_address(sbr, io, i)
      loc = vdai + SLOT_VALUE_SIZES_BEGIN
      
      pv = fetch_value_dimension(sbr, io, i)
      
      do j = 1, pv
         sizev(j) = rand_fetch_slot(sbr, io, loc)
         loc = loc + 1
      enddo
      
      return
      end
      
      
      integer function fetch_value_layout_kind(sbr, io, i)
C     Get the layout kind of ith value.
C     Fetch slot SLOT_VALUE_LAYOUT_KIND of item i of slot vm.
C     Fetch slot (vda(i) + SLOT_VALUE_LAYOUT_KIND, where vda(i) is the
C     value in slot (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, i
      
C     Local variables.
      integer vdai, pv, loc
      
C     External functions.
      integer fetch_communication_value_descriptor_address
      integer fetch_value_dimension, rand_fetch_slot
      
      vdai = fetch_communication_value_descriptor_address(sbr, io, i)
      pv = fetch_value_dimension(sbr, io, i)
      loc = vdai + SLOT_VALUE_LAYOUT_KIND(pv)
      
      fetch_value_layout_kind = rand_fetch_slot(sbr, io, loc)
      
      return
      end
      
      
      subroutine fetch_value_layout(sbr, io, i, layoutv)
C     Get the layout of ith value into layoutv.
C     Fetch slots SLOT_VALUE_LAYOUT_BEGIN..
C     (SLOT_VALUE_LAYOUT_BEGIN + pv - 1) of item i of slot vm into
C     layoutv, where pv is the dimension of the value. 
C     Fetch slots (vda(i) + SLOT_VALUE_LAYOUT_BEGIN)..
C     (vda(i) + SLOT_VALUE_LAYOUT_BEGIN + pv - 1) into layoutv, where
C     vda(i) is the value in slot
C     (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1)
C     and pv the value in slot SLOT_VALUE_DIMENSION.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, i
      
C     Output argument.
      integer layoutv(MAX_ARRAY_DIMENSION)
      
C     Local variables.
      integer vdai, pv, loc, j
      
C     External functions.
      integer fetch_communication_value_descriptor_address
      integer fetch_value_dimension, rand_fetch_slot
      
      vdai = fetch_communication_value_descriptor_address(sbr, io, i)
      pv = fetch_value_dimension(sbr, io, i)
      loc = vdai + SLOT_VALUE_LAYOUT_BEGIN(pv)
      
      do j = 1, pv
         layoutv(j) = rand_fetch_slot(sbr, io, loc)
         loc = loc + 1
      enddo
      
      return
      end
      
      
      integer function fetch_slot_value_lengths_begin(sbr, io, i)
C     Get the beginning slot of lengths of ith value.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, i
      
C     Local variables.
      integer tv, pv, lkv, slot
      
C     External functions.
      logical is_array_type, is_multi_d_type
      integer fetch_value_type, fetch_value_dimension
      integer fetch_value_layout_kind
      
      slot = SLOT_VALUE_SIZES_BEGIN
      
      tv = fetch_value_type(sbr, io, i)
      pv = fetch_value_dimension(sbr, io, i)
      
      if (is_array_type(tv)) then
         
         slot = SLOT_VALUE_LAYOUT_KIND(pv)
         
         if (is_multi_d_type(tv)) then
            
            lkv = fetch_value_layout_kind(sbr, io, i)
            slot = SLOT_VALUE_LAYOUT_BEGIN(pv)
            
            if (lkv .eq. PERMUTATION_LAYOUT_KIND) then
               slot = SLOT_VALUE_LENGTHS_BEGIN(pv)
            endif
            
         endif
         
      endif
      
      fetch_slot_value_lengths_begin = slot
      
      return
      end
      
      
      subroutine fetch_value_lengths(sbr, io, i, nl, lenv)
C     Get the lengths of ith value into lenv, and length of lenv into nl.
C     Fetch slots SLOT_VALUE_LAYOUT_BEGIN..
C     (SLOT_VALUE_LAYOUT_BEGIN + pv - 1) of item i of slot vm into
C     layoutv, where pv is the dimension of the value. 
C     Fetch slots (vda(i) + SLOT_VALUE_LAYOUT_BEGIN)..
C     (vda(i) + SLOT_VALUE_LAYOUT_BEGIN + pv - 1) into lenv, where
C     vda(i) is the value in slot
C     (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1)
C     and pv the value in slot SLOT_VALUE_DIMENSION.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, i
      
C     Output arguments.
      integer nl, lenv(MAX_STRING_ITEMS)
      
C     Local variables.
      integer vdai, slot, loc, tv, tuple_size, pv
      integer sizev(MAX_ARRAY_DIMENSION), j, k
      
C     External functions.
      logical pio_get_debug_mode
      logical is_simple_type, is_couple_type, is_triple_type
      logical is_array_type
      integer fetch_communication_value_descriptor_address
      integer fetch_value_type, fetch_value_dimension
      integer fetch_slot_value_lengths_begin, rand_fetch_slot
      
      vdai = fetch_communication_value_descriptor_address(sbr, io, i)
      slot = fetch_slot_value_lengths_begin(sbr, io, i)
      loc = vdai + slot
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In fetch_value_lengths: ',
     +        'vdai = ', vdai, ', slot = ', slot
      endif
      
      tv = fetch_value_type(sbr, io, i)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In fetch_value_lengths: ',
     +        'is_simple_type(tv) = ', is_simple_type(tv),
     +        ' is_couple_type(tv) = ', is_couple_type(tv),
     +        ' is_triple_type(tv) = ', is_triple_type(tv)
      endif
      
      if (is_simple_type(tv)) then
         tuple_size = 1
      else if (is_couple_type(tv)) then
         tuple_size = 2
      else if (is_triple_type(tv)) then
         tuple_size = 3
      else
         call error_unknown_typed_value_type(sbr, tv)
      endif
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In fetch_value_lengths: ',
     +        'tv = ', tv, ', tuple_size = ', tuple_size
      endif
      
      nl = tuple_size
      if (is_array_type(tv)) then
         
         pv = fetch_value_dimension(sbr, io, i)
         call fetch_value_sizes(sbr, io, i, sizev)
         
         if (pio_get_debug_mode()) then
            write(PIO_DEBUG, *)
     +           '[Pio/Fortran] In fetch_value_lengths: ',
     +           'pv = ', pv, ', sizev = ', (sizev(j), j=1,pv)
         endif
      
         do j = 1, pv
            nl = nl * sizev(j)
         enddo
         
      endif
      
      do k = 1, nl
         lenv(k) = rand_fetch_slot(sbr, io, loc)
         loc = loc + 1
      enddo
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In fetch_value_lengths: ',
     +        'lenv(1) = ', lenv(1), ', lenv(nl) = ', lenv(nl)
      endif
      
      return
      end
